
Class clsSQLite3Connection

    Instance m_FileName     As String
    Instance m_hDB          As Dword
    Instance m_LastError    As Long

    Class Method Destroy
        Me.Close_()
    End Method

    Class Method Close_() As Long
        If m_hDB > 0 Then
            sqlite3_close( m_hDB )
            m_hDB = 0
            Method = 1
        End If
    End Method

    Interface ISQLite3Connection: Inherit IUnknown

        Property Get Helpers() As ISQLite3ConnectionHelpers
            Property = Me
        End Property

        Property Get LastError() As Long
            Property = m_LastError
        End Property

        ' Opens a database, it will create one if not exist (sqlite behaviour).
        ' No filename for memory database.
        Method Open( Optional ByVal sFileName As String ) As Long
            If m_hDB > 0 Then Exit Method
            If Len( sFileName ) = 0 Then sFileName = ":memory:"
            m_LastError = sqlite3_open( ByVal StrPtr( sFileName ), m_hDB )
            Method = ( m_LastError = %SQLITE_OK And m_hDB > 0 )
        End Method

        Method Close() As Long
            Method = Me.Close_()
        End Method

        Property Get IsOpen() As Long
            Property = IsTrue( m_hDB )
        End Property

        Property Get FileName() As String
            Property = m_FileName
        End Property

    End Interface

    Interface ISQLite3ConnectionHelpers: Inherit IUnknown

        Property Get LibVersion() As String
            Local pszText As AsciiZ Ptr
            pszText = sqlite3_libversion()
            If pszText > 0 Then Property = @pszText
        End Property

        Property Get Database() As Dword
            Property = m_hDB
        End Property

        Method Execute( ByVal sSQL As String ) As Long
            Local oCmd As ISQLite3Command
            Local oConn As ISQLite3Connection
            oConn = Me
            oCmd = Class "clsSQLite3Command"
            oCmd.SQL = sSQL
            oCmd.Connection = oConn
            Method = oCmd.Execute()
            m_LastError = oCmd.LastError
            oConn = Nothing
            oCmd = Nothing
        End Method

        Method ExecuteReader( ByVal sSQL As String ) As ISQLite3Reader
            Local oCmd As ISQLite3Command
            Local oConn As ISQLite3Connection
            oConn = Me
            oCmd = Class "clsSQLite3Command"
            oCmd.SQL = sSQL
            oCmd.Connection = oConn
            Method = oCmd.ExecuteReader()
            m_LastError = oCmd.LastError
            oConn = Nothing
            oCmd = Nothing
        End Method

        Method Vacuum() As Long
            Method = Me.Execute( "VACUUM;" )
        End Method

        Method DropTable( ByVal sTableName As String ) As Long
            sTableName = Trim$( sTableName )
            If Len( sTableName ) = 0 Then Exit Method
            Method = Me.Execute( "DROP TABLE [" & sTableName & "];" )
        End Method

        Method GetSqliteMaster( Optional ByVal sType As String ) As ISQLite3Reader
            Local sSQL As String
            sSQL = "SELECT * FROM sqlite_master"
            If Len( sType ) Then sSQL = sSQL & " WHERE Type='" & sType & "'"
            sSQL = sSQL & " ORDER BY NAME;"
            Method = Me.ExecuteReader( sSQL )
        End Method

        Method GetTableInfo( ByVal sTableName As String ) As ISQLite3Reader
            Method = Me.ExecuteReader( "PRAGMA table_info(" & sTableName & ");" )
        End Method

        Property Get Last_Insert_RowID() As Quad
            Property = sqlite3_last_insert_rowid( m_hDB )
        End Property

    End Interface

End Class

Class clsSQLite3Command

    Instance m_oConnection      As ISQLite3Connection
    Instance m_Transactionmode  As Long
    Instance m_Parameters()     As Dword
    Instance m_SQL              As String
    Instance m_LastError        As Long

    Class Method Destroy
        Me.Clear_()
    End Method

    ' Removes the parameter objects currently referenced.
    ' Unf. redim preserve seems the fail on interfaces in arrays, solved via pointers.
    Class Method Clear_()

        Local nParameter As Long
        Local o As IUnknown

        If ArrayAttr( m_Parameters(), 0 ) Then
            For nParameter = 0 To UBound( m_Parameters )
                If m_Parameters( nParameter ) > 0 Then
                    Poke Dword, VarPtr( o ), m_Parameters( nParameter )
                    o = Nothing
                    m_Parameters( nParameter ) = 0
                End If
            Next
            Erase m_Parameters()
        End If

    End Method

    ' Prepares a statement handle.
    Class Method PrepareFromQuery_( ByVal sSQL As String, ByRef hDB As Dword, ByRef hstmt As Dword ) As Long
        If IsNothing( m_oConnection ) Then Exit Method
        hDB = m_oConnection.Helpers.Database
        If hDB = 0 Then Exit Method
        m_LastError = sqlite3_prepare_v2( hDB, ByVal StrPtr( sSQL ), Len( sSQL ), hstmt, ByVal 0& )
        If m_LastError <> %SQLITE_OK Then Exit Method
        Method = IsTrue( hstmt )
    End Method

    ' Returns the parameter index by value or name.
    ' 0 for failure.
    Class Method GetParameterIndexByVariant_( ByVal hstmt As Dword, ByVal vIndex As Variant ) As Long
        Local nIndex As Long
        Local T As String
        nIndex = Variant#( vIndex )
        If nIndex < 1 Then
            T = Variant$( vIndex )
            If Len( T ) Then nIndex = sqlite3_bind_parameter_index( hstmt, ByVal StrPtr( T ) )
        End If
        Method = nIndex
    End Method

    ' Set's or commit's a transaction.
    Class Method Transaction_( ByVal nMode As Long ) As Long

        Local hDB       As Dword
        Local sSQL      As String
        Local hstmt     As Dword

        Select Case nMode
        Case 0: sSQL = "BEGIN TRANSACTION;"
        Case 1: sSQL = "COMMIT TRANSACTION;"
        Case 2: sSQL = "ROLLBACK TRANSACTION;"
        Case Else: Exit Method
        End Select

        If Me.PrepareFromQuery_( sSQL, hDB, hstmt ) = 0 Then Exit Method
        m_LastError = sqlite3_step( hstmt )
        sqlite3_finalize( hstmt ): hstmt = 0
        Method = ( m_LastError = %SQLITE_OK )

    End Method

    ' Savepoints are substitutes for nested transactions, this feature is not tested yet.
    Class Method SavePoint_( ByVal TransactionName As String, ByVal nMode As Long ) As Long

        Local hDB       As Dword
        Local sSQL      As String
        Local hstmt     As Dword

        TransactionName = Trim$( TransactionName )
        If Len( TransactionName ) = 0 Then Exit Method

        Select Case nMode
        Case 0: sSQL = "savepoint " & TransactionName & ";"
        Case 1: sSQL = "release savepoint " & TransactionName & ";"
        Case Else: Exit Method
        End Select

        If Me.PrepareFromQuery_( sSQL, hDB, hstmt ) = 0 Then Exit Method
        m_LastError = sqlite3_step( hstmt )
        sqlite3_finalize( hstmt ): hstmt = 0
        Method = ( m_LastError = %SQLITE_OK )

    End Method

    ' Prepares a statement handle with parameters (if any, parameters are optional)
    Class Method Prepare_( ByVal cmd As ISQLite3Command ) As Long

        Local p                 As ISQLite3Parameter
        Local pint              As ISQLite3ParameterInt
        Local pdbl              As ISQLite3ParameterDouble
        Local pstr              As ISQLite3ParameterStr
        Local pblob             As ISQLite3ParameterBlob
        Local hDB               As Dword
        Local hstmt             As Dword
        Local nParameter        As Long
        Local sText             As String
        Local nParameterIndex   As Long

        If Me.PrepareFromQuery_( m_SQL, hDB, hstmt ) = 0 Then Exit Method

        For nParameter = 0 To cmd.Parameters.Count - 1

            p = cmd.Parameters.Item( nParameter )
            nParameterIndex = Me.GetParameterIndexByVariant_( hstmt, p.Name )

            If IsInterface( p, ISQLite3ParameterInt ) Then
                pint = p
                m_LastError = sqlite3_bind_int( hstmt, nParameterIndex, pint.Value )
                pint = Nothing
            ElseIf IsInterface( p, ISQLite3ParameterDouble ) Then
                pdbl = p
                m_LastError = sqlite3_bind_double( hstmt, nParameterIndex, pdbl.Value )
                pdbl = Nothing
            ElseIf IsInterface( p, ISQLite3ParameterStr ) Then
                pstr = p
                m_LastError = sqlite3_bind_text( hstmt, nParameterIndex, ByVal pstr.ValuePtr, -1, %SQLITE_STATIC )
                pstr = Nothing
            ElseIf IsInterface( p, ISQLite3ParameterBlob ) Then
                pblob = p
                m_LastError = sqlite3_bind_blob( hstmt, nParameterIndex, ByVal pblob.Value, pblob.Size, %SQLITE_STATIC )
                pblob = Nothing
            Else
                m_LastError = %SQLITE_MISUSE
            End If

            If m_LastError <> %SQLITE_OK Then Exit For

        Next

        If m_LastError = %SQLITE_OK Then
            Method = hstmt
        Else
            sqlite3_finalize( hstmt ): hstmt = 0
        End If

    End Method

    Interface ISQLite3Command: Inherit IUnknown

        Property Get Parameters() As ISQLite3CommandParameters
            Property = Me
        End Property

        Property Get Transaction() As ISQLite3CommandTransaction
            Property = Me
        End Property

        Property Get SavePoint() As ISQLite3CommandSavePoint
            Property = Me
        End Property

        Property Get LastError() As Long
            Property = m_LastError
        End Property

        Property Get Connection() As ISQLite3Connection
            Property = m_oConnection
        End Property

        Property Set Connection( ByVal value As ISQLite3Connection )
            m_oConnection = value
        End Property

        Property Get SQL() As String
            Property = m_SQL
        End Property

        Property Set SQL( ByVal value As String )
            m_SQL = value
        End Property

        ' Prepares a statement handle and returns a reader object for stepping.
        ' Use LastError to determine errors.
        Method ExecuteReader() As ISQLite3Reader
            Local r             As ISQLite3Reader
            Local b             As ISQLite3ReaderBackDoor
            Local hstmt         As Dword
            hstmt = Me.Prepare_( Me )
            If hstmt = 0 Then Exit Method
            r = Class "clsSQLite3Reader"
            b = r
            b.Initialize( m_oConnection, hstmt )
            Method = r
        End Method

        ' Prepares a statement handle and returns the nr of rows affected.
        ' Use LastError to determine errors.
        Method Execute() As Long

            Local hstmt         As Dword
            Local hDB           As Dword
            Local nOldChanges   As Long
            Local nNewChanges   As Long

            hstmt = Me.Prepare_( Me )
            If hstmt = 0 Then Exit Method

            hDB = m_oConnection.Helpers.Database
            nOldChanges = sqlite3_changes( hDB )
            m_LastError = sqlite3_step( hstmt )
            nNewChanges  = sqlite3_changes( hDB )
            Method = nNewChanges - nOldChanges
            sqlite3_finalize( hstmt ): hstmt = 0

        End Method

    End Interface

    Interface ISQLite3CommandParameters: Inherit IUnknown

        Method Clear() As Long
            Me.Clear_()
        End Method

        Property Get Count() As Long
            Property = ArrayAttr( m_Parameters(), 4 )
        End Property

        ' You can only add parameters to this collection.
        ' To discard one you'll need to clear and start over again.
        Method Add( ByVal Parameter As ISQLite3Parameter ) As Long

            If IsNothing( Parameter ) Then Exit Method
            Local nIndex As Long

            If ArrayAttr( m_Parameters(), 0 ) = 0 Then
                ReDim m_Parameters( 0 To 0 )
                nIndex = 0
            Else
                nIndex = ArrayAttr( m_Parameters(), 4 )
                ReDim Preserve m_Parameters( 0 To nIndex )
            End If

            m_Parameters( nIndex ) = ObjPtr( Parameter )
            Parameter.Addref()

        End Method

        ' Access a single parameter in this collection.
        Method Item( ByVal Index As Long ) As ISQLite3Parameter
            If Index < 0 Or Index > ArrayAttr( m_Parameters(), 4 ) - 1 Then Exit Method
            Local o As ISQLite3Parameter
            Poke Dword, VarPtr( o ), m_Parameters( Index )
            o.AddRef()
            Method = o
        End Method

    End Interface

    Interface ISQLite3CommandTransaction: Inherit IUnknown

        Method Begin() As Long
            Method = Me.Transaction_( 0 )
        End Method

        Method Commit() As Long
            Method = Me.Transaction_( 1 )
        End Method

        Method Rollback() As Long
            Method = Me.Transaction_( 2 )
        End Method

    End Interface

    Interface ISQLite3CommandSavePoint: Inherit IUnknown

        Method Begin( ByVal TransactionName As String ) As Long
            Method = Me.SavePoint_( TransactionName, 0 )
        End Method

        Method Commit( ByVal TransactionName As String ) As Long
            Method = Me.SavePoint_( TransactionName, 1 )
        End Method

    End Interface

End Class

' Parameters derive from a base class but their value properties differ from each other.
' The inheritance was needed for making the collection easier.

Class clsSQLite3Parameter

    Instance m_Name As Variant

    Interface ISQLite3Parameter: Inherit IUnknown

        ' The name property can be a parameter index numeric value for ? purposes or a name like @PARAM1.
        Property Get Name() As Variant
            Property = m_Name
        End Property

        Property Set Name( ByVal value As Variant )
            m_Name = value
        End Property

    End Interface

End Class

Class clsSQLite3ParameterInt

    Instance m_Value As Long

    Interface ISQLite3ParameterInt: Inherit clsSQLite3Parameter, ISQLite3Parameter

        Property Get Value() As Long
            Property = m_Value
        End Property

        Property Set Value( ByVal intvalue As Long )
            m_Value = intvalue
        End Property

    End Interface

End Class

Class clsSQLite3ParameterDouble

    Instance m_Value As Double

    Interface ISQLite3ParameterDouble: Inherit clsSQLite3Parameter, ISQLite3Parameter

        Property Get Value() As Double
            Property = m_Value
        End Property

        Property Set Value( ByVal dval As Double )
            m_Value = dval
        End Property

    End Interface

End Class

Class clsSQLite3ParameterStr

    Instance m_Value As String

    Interface ISQLite3ParameterStr: Inherit clsSQLite3Parameter, ISQLite3Parameter

        Property Get Value() As String
            Property = m_Value
        End Property

        Property Set Value( ByVal strvalue As String )
            m_Value = strvalue
        End Property

        Property Get ValuePtr() As Dword
            Property = StrPtr( m_Value )
        End Property

    End Interface

End Class

' The blob parameter does not cache its data, it simply passes an address for the data.
' The Size property indicates the size of the data.
Class clsSQLite3ParameterBlob

    Instance m_Value As Dword
    Instance m_Size As Dword

    Interface ISQLite3ParameterBlob: Inherit clsSQLite3Parameter, ISQLite3Parameter

        Property Get Value() As Dword
            Property = m_Value
        End Property

        Property Set Value( ByVal pval As Dword )
            m_Value = pval
        End Property

        Property Get Size() As Long
            Property = m_Size
        End Property

        Property Set Size( ByVal value As Long )
            m_Size = value
        End Property

    End Interface

End Class

Class clsSQLite3Reader

    Instance m_LastError    As Long
    Instance m_hstmt        As Dword
    Instance m_oConnection  As ISQLite3Connection
    Instance m_BOF          As Long
    Instance m_EOF          As Long

    Class Method Create
        m_BOF = -1
        m_EOF = -1
    End Method

    Class Method Destroy
        Me.Close_()
    End Method

    Class Method Close_() As Long
        If m_hstmt > 0 Then
            sqlite3_finalize( m_hstmt )
            m_hstmt = 0
            Method = 1
        End If
    End Method

    Interface ISQLite3Reader: Inherit IUnknown

        Property Get Fields() As ISQLite3ReaderFields
            Property = Me
        End Property

        Property Get LastError() As Long
            Property = m_LastError
        End Property

        Method Close() As Long
            Method = Me.Close_()
        End Method

        Property Get IsOpen() As Long
            Property = IsTrue( m_hstmt )
        End Property

        ' When BOF and EOF both are true then there are no records in this selection.
        Property Get Eof() As Long
            Property = m_EOF
        End Property

        Property Get Bof() As Long
            Property = m_BOF
        End Property

        ' When the query is executed this object will have advanced to the first record already.
        ' Use Read() to advance records.
        ' The step indicates the BOF and EOF or an error code.
        ' This method returns true on each succesful advance and false when there are no records anymore.
        Method Read() As Long

            m_LastError = sqlite3_step( m_hstmt )

            Select Case m_LastError
            Case %SQLITE_ROW:   m_BOF = 0: m_EOF = 0
            Case %SQLITE_DONE:  m_EOF = -1
            End Select

            Method = IsTrue( m_LastError = %SQLITE_ROW )

        End Method

        ' Sqlite is typeless, we use the value method to return every type for this field.
        ' This means everything will be returned as string, even double values.
        ' TODO: For specific cast use the other value methods.
        Method Value( ByVal nColumnIndex As Long ) As String
            Local nLen      As Long
            Local pData     As Dword
            pData = sqlite3_column_text( m_hstmt, nColumnIndex )
            If pData > 0 Then
                nlen = sqlite3_column_bytes( m_hstmt, nColumnIndex )
                If nlen < 1 Then Exit Method
                Method = Peek$( pData, nlen )
            End If
        End Method

    End Interface

    Interface ISQLite3ReaderFields: Inherit IUnknown

        Property Get Count() As Long
            Property = sqlite3_column_count( m_hstmt )
        End Property

        Method Item( ByVal Index As Long ) As ISQLite3ReaderField
            Local rf As ISQLite3ReaderField
            Local bd As ISQLite3ReaderFieldBackDoor
            Local r As ISQLite3Reader
            If Index < 0 Or ( Index + 1 ) > Me.Count Then Exit Method
            r = Me
            rf = Class "clsSQLite3ReaderField"
            bd = rf
            bd.Initialize( r, Index )
            Method = rf
        End Method

    End Interface

    ' Internal use only.
    Interface ISQLite3ReaderBackDoor: Inherit IUnknown

        Property Get Reader() As ISQLite3Reader
            Property = Me
        End Property

        Property Get hstmt() As Dword
            Property = m_hstmt
        End Property

        ' Initializes a recordset object, it claims the connection and statement handle.
        Method Initialize( ByVal oConnection As ISQLite3Connection, ByVal hstmt As Dword )
            m_oConnection = oConnection
            m_hstmt = hstmt
            Me.Reader.Read()
        End Method

    End Interface

End Class

Class clsSQLite3ReaderField

    Instance m_Reader   As ISQLite3Reader
    Instance m_Readerbd As ISQLite3ReaderBackDoor
    Instance m_Index    As Long

    Interface ISQLite3ReaderField: Inherit IUnknown

        Property Get Index() As Long
            Property = m_Index
        End Property

        Property Get Name() As String
            Local pszText As AsciiZ Ptr
            pszText = sqlite3_column_name( m_Readerbd.hstmt, m_Index )
            If pszText > 0 Then Property = @pszText
        End Property

        ' Returns %SQLITE_INTEGER and so on.
        Property Get Type() As Long
            Property = sqlite3_column_type( m_Readerbd.hstmt, m_Index )
        End Property

        Property Get TableName() As String
            Local pszText As AsciiZ Ptr
            pszText = sqlite3_column_table_name( m_Readerbd.hstmt, m_Index )
            If pszText > 0 Then Property = @pszText
        End Property

    End Interface

    ' Internal use only.
    Interface ISQLite3ReaderFieldBackDoor: Inherit IUnknown

        Method Initialize( ByVal oReader As ISQLite3Reader, ByVal nIndex As Long ) As Long
            m_Reader = oReader
            m_Readerbd = oReader
            m_Index = nIndex
        End Method

    End Interface

End Class
