Function CreateTable( ByVal sFileName As String, ByVal bDropTable As Long ) As Long

    Local hr    As Long
    Local oConn As ISQLite3Connection
    Local oCmd  As ISQLite3Command

    ' Instantiate a connection class.
    oConn = Class "clsSQLite3Connection"

    ' Open the database, it will create a new one if not exist.
    If oConn.Open( sFileName ) Then

        ' Instantiate a command object.
        oCmd = Class "clsSQLite3Command"

        ' Set a reference to the connection object.
        oCmd.Connection = oConn

        ' Permanently delete the table.
        If bDropTable Then
            oCmd.SQL = "DROP TABLE [TABLE1];"
            hr = oCmd.Execute()
        End If

        ' Set the SQL statement.
        oCmd.SQL = "CREATE TABLE [TABLE1] ( [ID] INTEGER PRIMARY KEY, [NAME] TEXT, [DoubleValue] DOUBLE, [MYDATA] BLOB );"

        ' Execute the statement, use the lasterror to determine an error.
        ' hr will return 0 for this example.
        hr = oCmd.Execute()

        ' Was there an error?
        If oCmd.LastError <> %SQLITE_DONE Then
            ? "Create table: " & Str$( hr ) & ", " & Str$( oCmd.LastError )
        End If

        ' Release the object and connection.
        oCmd = Nothing
        oConn = Nothing

    End If

End Function

Function FillTable( ByVal sFileName As String ) As Long

    Local hr        As Long
    Local oConn     As ISQLite3Connection
    Local oCmd      As ISQLite3Command
    Local p1        As ISQLite3ParameterStr
    Local p2        As ISQLite3ParameterDouble
    Local p3        As ISQLite3ParameterBlob
    Local nRecord   As Long
    Local sBlobdata As String

    ' Instantiate a connection class.
    oConn = Class "clsSQLite3Connection"

    ' Open the database, it will create a new one if not exist.
    If oConn.Open( sFileName ) Then

        ' Instantiate a command object.
        oCmd = Class "clsSQLite3Command"

        ' Set a reference to the connection object.
        oCmd.Connection = oConn

        ' We are going to fill the table with a lot of records, use a transaction to increase speed.
        oCmd.Transaction.Begin()

        For nRecord = 1 To 10

            ' The INSERT statement.
            oCmd.SQL = "INSERT INTO [TABLE1] ( [NAME], [DoubleValue], [MYDATA] ) VALUES( @NAME, @DBL, @BLOB );"

            ' On each new record we clear the command's parameters and use new instances.
            ' This may be time consuming but the test loop by itself is not common practise anyway.
            oCmd.Parameters.Clear()

            ' Create a string parameter.
            p1 = Class "clsSQLite3ParameterStr"
            ' Set the name for the value.
            p1.Name = "@NAME"
            ' Set the value.
            p1.Value = Format$( nRecord ) & " - " & Time$

            ' Add to the command.
            oCmd.Parameters.Add( p1 )

            p2 = Class "clsSQLite3ParameterDouble"
            p2.Name = "@DBL"
            p2.Value = 123456.78#
            oCmd.Parameters.Add( p2 )

            ' Create a blob parameter.
            p3 = Class "clsSQLite3ParameterBlob"
            p3.Name = "@BLOB"

            ' Prepare some 'blob' data.
            sBlobdata = Date$ & ", " & Time$ & ", " & Format$( nRecord )

            ' Blob binding is done via the address of the data and its size.
            p3.Value = StrPtr( sBlobdata )
            p3.Size = Len( sBlobdata )
            oCmd.Parameters.Add( p3 )

            ' Add the record.
            hr = oCmd.Execute()

            ' Exit loop on error.
            If oCmd.LastError <> %SQLITE_DONE Then Exit For

        Next

        ' If an error occured, do a rollback and show the user.
        If oCmd.LastError <> %SQLITE_DONE Then
            oCmd.Transaction.RollBack()
'            ? "Fill table: " & Str$( hr ) & ", " & Str$( oCmd.LastError )
        Else
            ' Commit all changes.
            oCmd.Transaction.Commit()
        End If

        oConn.Helpers.Vacuum()

        oCmd = Nothing
        oConn = Nothing

    End If

End Function

Function ReadTable( ByVal sFileName As String ) As Long

    Local oConn As ISQLite3Connection
    Local oCmd  As ISQLite3Command
    Local p1    As ISQLite3ParameterInt
    Local p2    As ISQLite3ParameterStr
    Local r     As ISQLite3Reader

    oConn = Class "clsSQLite3Connection"

    If oConn.Open( sFileName ) Then

        oCmd = Class "clsSQLite3Command"
        oCmd.Connection = oConn
        oCmd.SQL = "SELECT [ID], [NAME], [DoubleValue], [MYDATA] FROM [TABLE1] WHERE [ID] > @MYID AND [NAME] LIKE ? LIMIT 2;"

        p1 = Class "clsSQLite3ParameterInt"
        p1.Name = "@MYID"
        p1.Value = 2
        oCmd.Parameters.Add( p1 )

        p2 = Class "clsSQLite3ParameterStr"
        p2.Name = 2
        p2.Value = "%0%"
        oCmd.Parameters.Add( p2 )

        r = oCmd.ExecuteReader()
        If IsObject( r ) Then

            Do While Not r.Eof

                '? r.Value( 0 ) & ", " & r.Value( 1 ) & ", '" & r.Value( 2 ) & "'"
                r.Read()

            Loop

        End If

        r = Nothing
        oCmd = Nothing
        oConn = Nothing

    End If

End Function
